import math, pygame, sys
from OpenGL.GL import *
from OpenGL.GLU import *
from pygame.locals import *
from vec3 import *

w, h = 0, 0

def setup_ortho(x1, x2, y1, y2):
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glOrtho(x1, x2, y1, y2, -1, 1)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def setup_perspective():
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(64.0, float(w) / float(h), 0.1, 100.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init(resolution):
    pygame.init()
    pygame.display.set_mode(resolution, HWSURFACE | OPENGL | DOUBLEBUF)
    global w, h
    w, h = resolution
    glViewport(0, 0, w, h)

    #glEnable(GL_TEXTURE_2D)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glDisable(GL_DEPTH_TEST)
    glDisable(GL_LIGHTING)
    glClearColor(1, 1, 1, 0)
    #glClearColor(0, 0, 0, 0)
    glEnable(GL_LIGHT0)

def draw_grid():
    b = 30
    glBegin(GL_LINES)
    for x in xrange(-b, b):
        glVertex2f(x, -b)
        glVertex2f(x, +b)
        glVertex2f(-b, x)
        glVertex2f(+b, x)
    glEnd()
    
def draw_quad():
    glBegin(GL_QUADS)
    glTexCoord2f(0, 0)
    glVertex2f(-1, -1)
    glTexCoord2f(1, 0)
    glVertex2f(1, -1)
    glTexCoord2f(1, 1)
    glVertex2f(1, 1)
    glTexCoord2f(0, 1)
    glVertex2f(-1, 1)
    glEnd()

def draw_circle(pos, radius):
    glPushMatrix()
    glTranslatef(pos.x, pos.y, pos.z)
    glBegin(GL_TRIANGLE_FAN)
    glVertex2f(0, 0)
    steps = 36
    for t in range(steps + 1):
        glVertex2f(math.cos(math.pi * 2 / steps * t) * radius,
                   math.sin(math.pi * 2 / steps * t) * radius)
    glEnd()
    glPopMatrix()

def draw_vector(pos, direction):
    end = pos + direction
    glBegin(GL_LINES)
    glVertex2f(pos.x, pos.y)
    glVertex2f(end.x, end.y)
    glEnd()

def begin_frame():
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

def finish_frame():
    for event in pygame.event.get():
        if event.type == QUIT:
            sys.exit()  # TODO: asi ne tak naprudko..
        if event.type == KEYUP and event.key == K_ESCAPE:
            sys.exit()

    pygame.display.flip()

