from vec3 import *
from OpenGL.GL import *
import glsurface, math

class Seeker:
    def __init__(self):
        self.reset()

    def reset(self):
        self.forward = Vec3(0, 1, 0)
        self.up = Vec3(0, 0, 1)
        self.position = Vec3(0, 0, 0)
        self.set_unit_side_from_forward_and_up()
        
        self.speed = 0.0
        self.max_speed = 1.0
        self.max_force = 0.1
        self.radius = 0.5
        self.mass = 0.6
        self.running = True

    def velocity(self):
        return self.forward * self.speed

    def update_state(self, goal_pos, goal_radius):
        if self.running:
            base_dist = (self.position - goal_pos).length()
            if base_dist < (self.radius + goal_radius):
                print 'running == False'
                self.running = False

    def steer_for_seek(self, goal_pos):
        offset = goal_pos - self.position
        desired_velocity = offset.len_truncated(self.max_speed)
        return desired_velocity - self.velocity()

    def adjust_raw_steering_force(self, force):
        max_adjusted_speed = 0.2 * self.max_speed
        if self.speed > max_adjusted_speed or force == Vec3(0, 0, 0):
            return force
        range = self.speed / max_adjusted_speed
        cosine = lerp(1.0, -1.0, math.pow(range, 20))
        return force.limit_deviation_angle(cosine, self.forward)

    def regenerate_basis(self, new_velocity):
        if self.speed > 0:
            self.forward = new_velocity / self.speed
            self.set_unit_side_from_forward_and_up()
            self.up = self.side.cross(self.forward)
                               
    def set_unit_side_from_forward_and_up(self):
        self.side = self.forward.cross(self.up).normalized()

    def apply_steering_force(self, force, dt):
        adjusted_force = self.adjust_raw_steering_force(force)
        clipped_force = adjusted_force.len_truncated(self.max_force)
        new_acceleration = clipped_force / self.mass
        new_velocity = self.velocity()
        new_velocity += new_acceleration * dt
        new_velocity = new_velocity.len_truncated(self.max_speed)
        self.speed = new_velocity.length()
        self.position += new_velocity * dt
        self.regenerate_basis(new_velocity)

    def steering_for_seeker(self, goal_pos):
        seek = self.steer_for_seek(goal_pos)
        return seek.limit_deviation_angle(0.707, self.forward)

    def update(self, dt, goal_pos, goal_radius):
        if self.running:
            self.update_state(goal_pos, goal_radius)
            steer = self.steering_for_seeker(goal_pos)
            self.apply_steering_force(steer, dt)
