import math

def rad2deg(x): return x * 180.0 / math.pi
def deg2rad(x): return x * math.pi / 180.0
def lerp(a, b, t): return a + (b - a) * t

class Vec3:
    def __init__(self, *args):
        if not args:
            self.x = 0.0
            self.y = 0.0
            self.z = 0.0
            return
        if isinstance(args[0], Vec3):  # copy
            self.x = args[0].x
            self.y = args[0].y
            self.z = args[0].z
        else:
            self.x = float(args[0])
            self.y = float(args[1])
            self.z = float(args[2])

    def __eq__(self, v):
        return self.x == v.x and self.y == v.y and self.z == v.z

    def __ne__(self, v):
        return not self == v
        
    def __neg__(self):
        return Vec3(-self.x, -self.y, -self.z)

    def __add__(self, v):
        return Vec3(self.x + v.x, self.y + v.y, self.z + v.z)

    def __iadd__(self, v):
        self.x += v.x
        self.y += v.y
        self.z += v.z
        return self

    def __sub__(self, v):
        return Vec3(self.x - v.x, self.y - v.y, self.z - v.z)

    def __isub__(self, v):
        self.x -= o.x
        self.y -= o.y
        self.z -= o.z
        return self

    def __mul__(self, s):
        if isinstance(s, Vec3):
            return Vec3(self.x * s.x, self.y * s.y, self.z * s.z)
        return Vec3(self.x * s, self.y * s, self.z * s)

    def __imul__(self, s):
        self.x *= s
        self.y *= s
        self.z *= s
        return self

    def __div__(self, s):
        return Vec3(self.x / s, self.y / s, self.z / s)

    def __idiv__(self, s):
        self.x /= s
        self.y /= s
        self.z /= s
        return self

    def length2(self):
        return self.dot(self)
        
    def length(self):
        return math.sqrt(self.length2())

    def normalized(self):
        l = self.length()
        return Vec3(self.x / l, self.y / l, self.z / l)

    def dot(self, v):
        return self.x * v.x + self.y * v.y + self.z * v.z

    def cross(self, v):
        return Vec3(self.y * v.z - self.z * v.y,
                    self.z * v.x - self.x * v.z,
                    self.x * v.y - self.y * v.x)

    def __repr__(self):
        return 'Vec3: [' + str(self.x) + ', ' + str(self.y) + ', ' + str(self.z) + ']'

    __str__ = __repr__

    def angle(self, other):
        s = math.sqrt(self.length2() * other.length2())
        if s == 0.0: return 0.0
        return math.acos(self.dot(other) / s)

    def parallel_comp(self, unit_basis):
        return unit_basis * (self * unit_basis)

    def perp_comp(self, unit_basis):
        return self - self.parallel_comp(unit_basis)

    def len_truncated(self, max_len):
        max2 = max_len**2
        vec2 = self.length2()
        if vec2 <= max2:
            return Vec3(self)
        return self * (max_len / math.sqrt(vec2))

    def limit_deviation_angle(self, angle, basis):
        slen = self.length()
        if slen == 0.0: return Vec3(self)

        direction = self / slen
        angle_with = direction.dot(basis)
        if angle_with >= angle: return Vec3(self)

        unit_perp = self.perp_comp(basis).normalized()
        perp_dist = math.sqrt(1 - (angle * angle_with))
        c0 = basis * angle
        c1 = unit_perp * perp_dist
        return (c0 + c1) * slen

        
